# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: node_service.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12node_service.proto\x12\x04node\"U\n\tErrorInfo\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x14\n\x0c\x64\x65tails_json\x18\x03 \x01(\t\x12\x13\n\x0bstack_trace\x18\x04 \x01(\t\"\xaa\x02\n\x0bNodeMessage\x12\n\n\x02id\x18\x01 \x01(\t\x12\x1f\n\x04type\x18\x02 \x01(\x0e\x32\x11.node.MessageType\x12\x0e\n\x06method\x18\x03 \x01(\t\x12\x0e\n\x06source\x18\x04 \x01(\t\x12\x0e\n\x06target\x18\x05 \x01(\t\x12\x0f\n\x07payload\x18\x06 \x01(\x0c\x12\x16\n\x0e\x63orrelation_id\x18\x07 \x01(\t\x12\x11\n\ttimestamp\x18\x08 \x01(\x03\x12\x31\n\x08metadata\x18\t \x03(\x0b\x32\x1f.node.NodeMessage.MetadataEntry\x12\x1e\n\x05\x65rror\x18\n \x01(\x0b\x32\x0f.node.ErrorInfo\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x07\n\x05\x45mpty*y\n\x0bMessageType\x12\x1c\n\x18MESSAGE_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14MESSAGE_TYPE_REQUEST\x10\x01\x12\x19\n\x15MESSAGE_TYPE_RESPONSE\x10\x02\x12\x17\n\x13MESSAGE_TYPE_NOTIFY\x10\x03\x32\x65\n\x0bNodeService\x12,\n\x04\x43\x61ll\x12\x11.node.NodeMessage\x1a\x11.node.NodeMessage\x12(\n\x06Notify\x12\x11.node.NodeMessage\x1a\x0b.node.Emptyb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'node_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_NODEMESSAGE_METADATAENTRY']._options = None
  _globals['_NODEMESSAGE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGETYPE']._serialized_start=425
  _globals['_MESSAGETYPE']._serialized_end=546
  _globals['_ERRORINFO']._serialized_start=28
  _globals['_ERRORINFO']._serialized_end=113
  _globals['_NODEMESSAGE']._serialized_start=116
  _globals['_NODEMESSAGE']._serialized_end=414
  _globals['_NODEMESSAGE_METADATAENTRY']._serialized_start=367
  _globals['_NODEMESSAGE_METADATAENTRY']._serialized_end=414
  _globals['_EMPTY']._serialized_start=416
  _globals['_EMPTY']._serialized_end=423
  _globals['_NODESERVICE']._serialized_start=548
  _globals['_NODESERVICE']._serialized_end=649
# @@protoc_insertion_point(module_scope)
